import { BaseRepository } from '#application/base/baseRepository'
import { UUID } from '#application/types'
import { User } from '#entities/user'

class UserRepository extends BaseRepository {
  async getById(id: UUID) {
    try {
      const repository = this.getEntityManager().getRepository(User)
      return await repository.findOne({ id })
    } catch (error: any) {
      this.logger.error(`Failed to get user by ID: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getByUsername(username: string) {
    try {
      const repository = this.getEntityManager().getRepository(User)
      return await repository.findOne({ username })
    } catch (error: any) {
      this.logger.error(`Failed to get user by username: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getByEmail(email: string) {
    try {
      const repository = this.getEntityManager().getRepository(User)
      return await repository.findOne({ email })
    } catch (error: any) {
      this.logger.error(`Failed to get user by email: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }
}

export default UserRepository
